@@ -11,11 +11,14 @@ fit_sbm_const = function(	trees, 								# either a single tree in phylo format,
 							tip_longitudes, 					# either a 1D vector of size Ntips (if trees[] is a single tree) or a list of 1D vectors (if trees[] is a list of trees), listing geographical longitudes of the tips (in decimal degrees) of each tree
 							radius,								# numeric, radius to assume for the sphere (e.g. Earth). Use this e.g. if you want to hange the units in which diffusivity is estimated. Earth's mean radius is about 6371e3 m.
 							phylodistance_matrixes	= NULL,		# optional list of length Ntrees, each element being a phylodistance matrix for the tips of a tree. Hence phylodistance_matrixes[[tr]][i,j] is the phylogenetic (patristic) distance between tips i & j in tree tr. Can be used to specify distances between tips regardless of the edge lengths in the trees (i.e., trees are only used for topology). Some phylodistances may be NA or NaN; the corresponding tip pairs will be omitted from the fitting.
+							clade_states			= NULL,		# optional, either an integer vector of length Ntips+Nnodes (if trees[] is a single tree) or a list of 1D vectors (if trees[] is a list of trees), specifying the discrete "state" of each tip and node in each tree. This can be used to limit independent contrasts to tip pairs whose total number of state-transitions (along their shortest path) is zero.
 							planar_approximation	= FALSE,	# logical, specifying whether the estimation formula should be based on a planar approximation of Earth's surface, i.e. geodesic angles are converted to distances and then those are treated as if they were Euclideanon a 2D plane. This approximation substantially increases the speed of computations.
 							only_basal_tip_pairs	= FALSE,	# logical, specifying whether only immediate sister tips should be considered, i.e. tip pairs with at most 2 edges between the two tips
 							only_distant_tip_pairs	= FALSE,	# logical, whether to only consider tip pairs located at distinct geographic locations
 							min_MRCA_time			= 0,		# numeric, specifying the minimum allowed height (distance from root) of the MRCA of sister tips considered in the fitting. In other words, an independent contrast is only considered if the two sister tips' MRCA has at least this distance from the root. Set min_MRCA_time=0 to disable this filter.
 							max_MRCA_age			= Inf,		# numeric, specifying the maximum allowed age (distance from youngest tip) of the MRCA of sister tips considered in the fitting. In other words, an independent contrast is only considered if the two sister tips' MRCA has at most this age (time to present). Set max_MRCA_age=Inf to disable this filter.
+							no_state_transitions	= FALSE,	# if TRUE, only tip pairs without state transitions along their shortest paths are considered. In particular, only tips in the same state are considered. Requires that clade_states[] is provided.
+							only_state				= NULL,		# optional integer, specifying the state in which tip pairs (and their connecting ancestors) must be in order to be considered. Requires that clade_states[] is provided.
 							min_diffusivity			= NULL,		# numeric, specifying the lower bound of allowed diffusivities. If omitted, it will be automatically chosen. Only relevant if planar_approximation==FALSE.
 							max_diffusivity			= NULL,		# numeric, specifying the upper bound of allowed diffusivities. If omitted, it will be automatically chosen. Only relevant if planar_approximation==FALSE.
 							Nbootstraps				= 0,		# integer, number of boostraps to perform. If <=0, no boostrapping is performed.
@@ -36,6 +39,9 @@ fit_sbm_const = function(	trees, 								# either a single tree in phylo format,
 		if((!is.null(phylodistance_matrixes)) && (!(("list" %in% class(phylodistance_matrixes)) && (length(phylodistance_matrixes)==1)))){
 			phylodistance_matrixes = list(phylodistance_matrixes)
 		}
+		if((!is.null(clade_states)) && (!(("list" %in% class(clade_states)) && (length(clade_states)==1)))){
+			clade_states = list(clade_states)
+		}
 	}else if("list" %in% class(trees)){
 		# trees[] is a list of trees
 		Ntrees = length(trees)
@@ -63,12 +69,23 @@ fit_sbm_const = function(	trees, 								# either a single tree in phylo format,
 				return(list(success=FALSE, error="Unknown data type for phylodistance_matrixes: Exected a list of matrixes"))
 			}
 		}
+		if(!is.null(clade_states)){
+			if("list" %in% class(clade_states)){
+				if(length(clade_states)!=Ntrees) return(list(success=FALSE,error=sprintf("Input list of clade_states has length %d, but should be of length %d (number of trees)",length(clade_states),Ntrees)))
+			}else if(("numeric" %in% class(clade_states)) || ("integer" %in% class(clade_states))){
+				if(Ntrees!=1) return(list(success=FALSE,error=sprintf("Input clade_states was given as a single vector, but expected a list of %d vectors (number of trees)",Ntrees)))
+				if(length(clade_states)!=length(trees[[1]]$tip.label)) return(list(success=FALSE,error=sprintf("ERROR: Input clade_states was given as a single vector of length %d, but expected length %d (number of tips in the input tree)",length(clade_states),length(trees[[1]]$tip.label))))
+				clade_states = list(clade_states)
+			}
+		}
 	}else{
 		return(list(success=FALSE,error=sprintf("Unknown data format '%s' for input trees[]: Expected a list of phylo trees or a single phylo tree",class(trees)[1])))
 	}
 	if(is.null(min_diffusivity) || is.na(min_diffusivity)) min_diffusivity = NaN;
 	if(is.null(max_diffusivity) || is.na(max_diffusivity)) max_diffusivity = NaN;
 	if(is.null(Nbootstraps) || is.na(Nbootstraps) || (Nbootstraps<0)) Nbootstraps = 0;
+	if((!is.null(only_state)) && is.null(clade_states)) return(list(success=FALSE, error="Missing clade_states[], needed when only_state is specified"))
+	if(no_state_transitions && is.null(clade_states)) return(list(success=FALSE, error="Missing clade_states[], needed when no_state_transitions=TRUE"))
 	if(min_MRCA_time<0) min_MRCA_time = 0
 
 	# loop through the trees and extract independet contrasts between sister clades
@@ -76,9 +93,10 @@ fit_sbm_const = function(	trees, 								# either a single tree in phylo format,
 	geodistances		= numeric()
 	tip_pairs_per_tree	= vector(mode="list", Ntrees)
 	for(i in 1:Ntrees){
-		tree = trees[[i]]
-		tip_latitudes_this_tree  = tip_latitudes[[i]]
-		tip_longitudes_this_tree = tip_longitudes[[i]]
+		tree 						= trees[[i]]
+		tip_latitudes_this_tree  	= tip_latitudes[[i]]
+		tip_longitudes_this_tree 	= tip_longitudes[[i]]
+		clade_states_this_tree 		= clade_states[[i]]
 		
 		# make sure tree does not have multifurcations
 		tree = multifurcations_to_bifurcations(tree)$tree
@@ -115,10 +133,17 @@ fit_sbm_const = function(	trees, 								# either a single tree in phylo format,
 			phylodistances_this_tree = phylodistance_matrix_this_tree[tip_pairs]
 		}
 
-		# omit tip pairs with zero phylogenetic distance, because in that case the likelihood density is pathological
+		# filter tip pairs
+		# in particular, omit tip pairs with zero phylogenetic distance, because in that case the likelihood density is pathological
 		# also omit tip pairs located at the same geographic location, if requested
+		# also omit tips whose shortest path includes state transitions, if requested
 		keep_pair = (is.finite(phylodistances_this_tree) & (phylodistances_this_tree>0))
 		if(only_distant_tip_pairs) keep_pair = keep_pair & (geodistances_this_tree>0)
+		if(no_state_transitions || (!is.null(only_state))){
+			Ntransitions = count_transitions_between_clades(tree=tree, A=tip_pairs[,1], B=tip_pairs[,2], states=clade_states_this_tree, check_input=TRUE)
+			if(no_state_transitions) keep_pair = keep_pair & (Ntransitions==0)
+			if(!is.null(only_state)) keep_pair = keep_pair & (Ntransitions==0) & (clade_states_this_tree[tip_pairs[,1]]==only_state) & (clade_states_this_tree[tip_pairs[,2]]==only_state)
+		}
 		tip_pairs					= tip_pairs[keep_pair,,drop=FALSE]
 		phylodistances_this_tree 	= phylodistances_this_tree[keep_pair]
 		geodistances_this_tree		= geodistances_this_tree[keep_pair]
@@ -132,7 +157,7 @@ fit_sbm_const = function(	trees, 								# either a single tree in phylo format,
 	NC = length(phylodistances)
 	if(NC==0) return(list(success=FALSE, error="No valid tip pairs left for extracting independent contrasts"))
 	if(all(geodistances==0)) return(list(success=FALSE, error="The geodesic distances of all independent contrasts are zero"))
-		
+
 	bootstrap_diffusivities = NULL
 	if(planar_approximation){
 		# ignore spherical structure and simply translate geodesic_angles to geodesic distances, then fit Brownian motion on a plane with isotropic diffusivity matrix
@@ -217,6 +242,7 @@ fit_sbm_const = function(	trees, 								# either a single tree in phylo format,
 		QQplot = cbind(quantile(geodistances, probs=probs, na.rm=TRUE, type=8), quantile(sim_geodistances, probs=probs, na.rm=TRUE, type=8))
 	}
 	
+	
 	#####################################
 
 	return(list(success					= TRUE,
